@@ -40,7 +40,6 @@ else:
     from mock import MagicMock
 
 PATCH_METHOD = 'botocore.operation.Operation.call'
-SESSION_PATCH_METHODD = 'botocore.session.get_session'
 
 
 class OldStyleModel(Model):
@@ -456,6 +455,38 @@ class ModelTestCase(TestCase):
             args = req.call_args[1]
             self.assertEqual(args, params)
 
+        with patch(PATCH_METHOD) as req:
+            req.return_value = HttpOK(), None
+            item.delete(user_id='bar', email__contains='@', conditional_operator='AND')
+            params = {
+                'key': {
+                    'user_id': {
+                        'S': 'bar'
+                    },
+                    'user_name': {
+                        'S': 'foo'
+                    }
+                },
+                'expected': {
+                    'email': {
+                        'AttributeValueList': [
+                            {'S': '@'}
+                        ],
+                        'ComparisonOperator': 'CONTAINS'
+                    },
+                    'user_id': {
+                        'Value': {
+                            'S': 'bar'
+                        }
+                    }
+                },
+                'conditional_operator': 'AND',
+                'return_consumed_capacity': 'TOTAL',
+                'table_name': 'UserModel'
+            }
+            args = req.call_args[1]
+            self.assertEqual(args, params)
+
     def test_update_item(self):
         """
         Model.update_item
@@ -506,7 +537,7 @@ class ModelTestCase(TestCase):
                     }
                 }
             }
-            item.update_item('views', 10, action='add', user_name='foo')
+            item.update_item('views', 10, action='add', user_name='foo', email__not_contains='@')
             args = req.call_args[1]
             params = {
                 'table_name': 'SimpleModel',
@@ -519,7 +550,13 @@ class ModelTestCase(TestCase):
                 'expected': {
                     'user_name': {
                         'Value': {'S': 'foo'}
-                    }
+                    },
+                    'email': {
+                        'AttributeValueList': [
+                            {'S': '@'}
+                        ],
+                        'ComparisonOperator': 'NOT_CONTAINS'
+                    },
                 },
                 'attribute_updates': {
                     'views': {
@@ -630,7 +667,7 @@ class ModelTestCase(TestCase):
 
         with patch(PATCH_METHOD) as req:
             req.return_value = HttpOK({}), {}
-            item.save(user_name='bar')
+            item.save(email__exists=False, zip_code__null=False)
             args = req.call_args[1]
             params = {
                 'item': {
@@ -647,9 +684,51 @@ class ModelTestCase(TestCase):
                         'S': u'foo'
                     },
                 },
+                'expected': {
+                    'email': {
+                        'Exists': False
+                    },
+                    'zip_code': {
+                        'ComparisonOperator': 'NOT_NULL'
+                    }
+                },
+                'return_consumed_capacity': 'TOTAL',
+                'table_name': 'UserModel'
+            }
+            self.assertEqual(args, params)
+
+        with patch(PATCH_METHOD) as req:
+            req.return_value = HttpOK({}), {}
+            item.save(user_name='bar', zip_code__null=True, email__contains='@', conditional_operator='OR')
+            args = req.call_args[1]
+            params = {
+                'item': {
+                    'callable_field': {
+                        'N': '42'
+                    },
+                    'email': {
+                        'S': u'needs_email'
+                    },
+                    'user_id': {
+                        'S': u'bar'
+                    },
+                    'user_name': {
+                        'S': u'foo'
+                    },
+                },
+                'conditional_operator': 'OR',
                 'expected': {
                     'user_name': {
                         'Value': {'S': 'bar'}
+                    },
+                    'zip_code': {
+                        'ComparisonOperator': 'NULL'
+                    },
+                    'email': {
+                        'ComparisonOperator': 'CONTAINS',
+                        'AttributeValueList': [
+                            {'S': '@'}
+                        ]
                     }
                 },
                 'return_consumed_capacity': 'TOTAL',
@@ -826,6 +905,52 @@ class ModelTestCase(TestCase):
             for item in CustomAttrNameModel.query('bar', overidden_user_name__eq='foo'):
                 self.assertIsNotNone(item)
 
+        with patch(PATCH_METHOD) as req:
+            items = []
+            for idx in range(10):
+                item = copy.copy(GET_MODEL_ITEM_DATA.get(ITEM))
+                item['user_id'] = {STRING_SHORT: 'id-{0}'.format(idx)}
+                items.append(item)
+            req.return_value = HttpOK({'Items': items}), {'Items': items}
+            queried = []
+            for item in UserModel.query(
+                    'foo',
+                    user_id__begins_with='id',
+                    email__contains='@',
+                    picture__null=False):
+                queried.append(item._serialize())
+            params = {
+                'key_conditions': {
+                    'user_id': {
+                        'AttributeValueList': [
+                            {'S': 'id'}
+                        ],
+                        'ComparisonOperator': 'BEGINS_WITH'
+                    },
+                    'user_name': {
+                        'AttributeValueList': [
+                            {'S': 'foo'}
+                        ],
+                        'ComparisonOperator': 'EQ'
+                    }
+                },
+                'query_filter': {
+                    'email': {
+                        'AttributeValueList': [
+                            {'S': '@'}
+                        ],
+                        'ComparisonOperator': 'CONTAINS'
+                    },
+                    'picture': {
+                        'ComparisonOperator': 'NULL'
+                    }
+                },
+                'return_consumed_capacity': 'TOTAL',
+                'table_name': 'UserModel'
+            }
+            self.assertEqual(params, req.call_args[1])
+            self.assertTrue(len(queried) == len(items))
+
     def test_scan(self):
         """
         Model.scan
@@ -873,6 +998,29 @@ class ModelTestCase(TestCase):
             for item in UserModel.scan():
                 self.assertIsNotNone(item)
 
+        with patch(PATCH_METHOD) as req:
+            items = []
+            for idx in range(10):
+                item = copy.copy(GET_MODEL_ITEM_DATA.get(ITEM))
+                item['user_id'] = {STRING_SHORT: 'id-{0}'.format(idx)}
+                items.append(item)
+            req.return_value = HttpOK({'Items': items}), {'Items': items}
+            for item in UserModel.scan(user_id__contains='tux'):
+                self.assertIsNotNone(item)
+            params = {
+                'return_consumed_capacity': 'TOTAL',
+                'scan_filter': {
+                    'user_id': {
+                        'AttributeValueList': [
+                            {'S': 'tux'}
+                        ],
+                        'ComparisonOperator': 'CONTAINS'
+                    }
+                },
+                'table_name': 'UserModel'
+            }
+            self.assertEquals(params, req.call_args[1])
+
     def test_get(self):
         """
         Model.get
@@ -1110,20 +1258,20 @@ class ModelTestCase(TestCase):
             LocalIndexedModel._get_meta_data()
 
         queried = []
-        # user_id not valid
-        with self.assertRaises(ValueError):
-            for item in IndexedModel.email_index.query('foo', user_id__between=['id-1', 'id-3']):
-                queried.append(item._serialize().get(RANGE))
+        with patch(PATCH_METHOD) as req:
+            with self.assertRaises(ValueError):
+                for item in IndexedModel.email_index.query('foo', user_id__between=['id-1', 'id-3']):
+                    queried.append(item._serialize().get(RANGE))
 
-        # startswith not valid
-        with self.assertRaises(ValueError):
-            for item in IndexedModel.email_index.query('foo', user_name__startswith='foo'):
-                queried.append(item._serialize().get(RANGE))
+        with patch(PATCH_METHOD) as req:
+            with self.assertRaises(ValueError):
+                for item in IndexedModel.email_index.query('foo', user_name__startswith='foo'):
+                    queried.append(item._serialize().get(RANGE))
 
-        # name not valid
-        with self.assertRaises(ValueError):
-            for item in IndexedModel.email_index.query('foo', name='foo'):
-                queried.append(item._serialize().get(RANGE))
+        with patch(PATCH_METHOD) as req:
+            with self.assertRaises(ValueError):
+                for item in IndexedModel.email_index.query('foo', name='foo'):
+                    queried.append(item._serialize().get(RANGE))
 
         with patch(PATCH_METHOD) as req:
             items = []
@@ -1374,7 +1522,8 @@ class ModelTestCase(TestCase):
             })
             schema = LocalIndexedModel.email_index.get_schema()
             args = req.call_args[1]
-            self.assertEqual(schema, collections.OrderedDict(params))
+            self.assert_dict_lists_equal(schema['attribute_definitions'], params['attribute_definitions'])
+            self.assert_dict_lists_equal(schema['key_schema'], params['key_schema'])
             self.assertTrue('ProvisionedThroughput' not in args['local_secondary_indexes'][0])
 
     def test_projections(self):
